//**********************************************************************
// 	Project: 
//	File: sdm_exp.v
// 	Description: test SDM 24 bits （GW_2A_18）
//	Author: WX
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	| WX    	| xx/xx/2021	| Initial version
//**********************************************************************


module sdm_exp (
// --- sym ---
	input    SYM_CLK,
// --- KEY ---
	input    KEY1,
    output pdm_clk,
	output	dout,
    output  lock

);

//**********************************************************************
// --- Parameter
//**********************************************************************
parameter DIN_WIDTH = 18;
parameter COEFF_WIDTH = 18;
parameter NUM_CYCLE=5;
//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************

    wire sys_clk;// wire clk_100M;
    assign sys_clk = SYM_CLK;
    wire clk;
	wire rstn;


    assign rstn = KEY1;

    wire	inpvalid;
    reg [3:0] cnt_cycle;

    reg coeff_wr=0;
    reg [15:0]coeff_addr=0;
    reg [COEFF_WIDTH-1:0] coeff_data=0;
    

    wire fifo_Full;
    wire fifo_Empty;
    wire lock_20,lock_100;
    wire signed [17:0] in_pcm_data;
    reg signed [7:0] in_pcm_data_8;
//**********************************************************************
// --- Initial
//**********************************************************************
assign lock = lock_20&&lock_100;
assign inpvalid  =      (lock_20&&lock_100&&( cnt_cycle == 1)) ? 1'b1 : 1'b0 ;

    always @(posedge clk or negedge rstn) begin
        if(!rstn) 
            cnt_cycle <= NUM_CYCLE-1;
        else if(cnt_cycle == NUM_CYCLE-1 ) 
            cnt_cycle <= 0;
        else if (lock_100&lock_20)
            cnt_cycle <= cnt_cycle + 1;
        else 
            cnt_cycle <= 0;
    end
    always @(posedge clk or negedge rstn) begin
        if(!rstn) 
            in_pcm_data_8 <= 0;
        else if (lock_100&lock_20)
            in_pcm_data_8 <= in_pcm_data_8 + 1;
        else
            in_pcm_data_8 <= 0;
    end

assign in_pcm_data={10'b0,in_pcm_data_8};
//**********************************************************************
// --- Module: top 
// --- Description:	top module of project
//**********************************************************************
SDM_Top Sigma_Delta_Modulator(
.clk    (clk),
.rstn  (rstn),
.coeff_wr     (coeff_wr),
.coeff_addr     (coeff_addr),
.coeff_in       (coeff_data),

.in_pcm_valid   (inpvalid),
.in_pcm_data   (in_pcm_data),
.in_pdm_clk   (pdm_clk),
.out_pdm_data   (dout),

.fifo_RdEn (1'b1),
.fifo_Empty (fifo_Empty),
.fifo_Full  (fifo_Full)
);




Gowin_rPLL_20 u_PLL_20(
    .clkout(pdm_clk), //output clkout 20MHz
    .clkin(sys_clk), //input clkin  50Mhz
    .reset(!rstn),
    .lock(lock_20)
);
Gowin_rPLL_100 u_PLL_100(
    .clkout(clk), //output clkout 100MHz
    .clkin(sys_clk), //input clkin  50Mhz
    .reset(!rstn),
    .lock(lock_100)
);

endmodule
